/*****************************************************************************/
/*                                                                           */
/*  HyperPanel OS                       File "src/space_iocont_stm32m4.c"    */
/*  Board - Configurable static data for stm32m4.                            */
/*                                                                           */
/*  Author : JYA/TE                     Last modification : 05/29/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


    The module "space_iocont.c" contains all the static RAM data of the VMIO
(IO container). These data are arrays wich size can be different, depends on
the target. Each array size is a define which value is define in a specific
"space_xx.h" file. By using different copies of the "space_xx.h" file, we can
customize the sizeo of the RAM arrays.
    
*/


#include <stdtyp.l>                    /* Hyperpanel definitions & constants */
#include <aut.l>                       /* AUT_VMIO_FIRST                     */
                                       /*                                    */
#include <aut.h>                       /* S_evt                              */
#include <board.h>                     /* Needed by req_lib.h                */
#include <vmio.h>                      /* A                                  */
#include <vmiodbg.h>                   /* Dura_crit_it                       */
#include <vmk.h>                       /* Route                              */

#define SPACE_IOCONT_XYZ_C

/*  3.1  VMIO tables              
    ----------------
*/

#define MAXAUT_VMIO          17        /* Maximum number of FSM's            */
#define MAXEVT_VMIO          16        /* Internal waiting queue size        */
#define NTOMAX_VMIO           8        /* Maximum number of Time-out's       */
#define MAXRTE_VMIO           8        /* Maximum number of event's routes   */

#define MAX_NOTE_EVT         10
#define MAX_ERR_EVT           5

#include <vmio.h>                      /* VMIO tables                        */

/*  3.2  "vmio_desc[] table, needed by "init_vmio"
    ----------------------------------------------
*/

extern const Vmio_desc rslv_vmio_desc; /* AUT_RESOLVE                        */
extern const Vmio_desc mem_vmio_desc ; /* AUT_MEM_VMIO                       */
extern const Vmio_desc gpio_vmio_desc; /* AUT_GPIO                           */
extern const Vmio_desc i2c_vmio_desc ; /* AUT_I2C                            */
extern const Vmio_desc asy_vmio_desc ; /* AUT_ASY                            */
extern const Vmio_desc arp_vmio_desc ; /* AUT_ARP                            */
extern const Vmio_desc ip_vmio_desc  ; /* AUT_IP                             */
extern const Vmio_desc dns_vmio_desc ; /* AUT_DNS                            */
extern const Vmio_desc vmioth_desc   ; /* AUT_VMIOTH                         */

const Vmio_desc *vmio_desc[MAXAUT_VMIO] =  /* Table of Vmio_desc addresses   */
  BEGIN                                /* ---------------------------------- */
    &rslv_vmio_desc                 ,  /* 0  AUT_RESOLVE - Must be 0         */
    &mem_vmio_desc                  ,  /* 1  AUT_RESOLVE - Must be 0         */
    &gpio_vmio_desc                 ,  /* 2  AUT_GPIO                        */
    &i2c_vmio_desc                  ,  /* 3  AUT_I2C                         */
    &asy_vmio_desc                  ,  /* 4  AUT_ASY                         */
    &vmioth_desc                    ,  /* 5  AUT_VMIOTH                      */
  END                               ;  /* ---------------------------------- */

/* 3.3  "req_lib.c" tables
   -----------------------
*/

#define PKG_req_lib                   /* Use the PKG_req_lib package         */

#define NBJOB                64       /* Size of "staticjob[]"               */

#include <req_lib.h>


/* 3.4  Interrupt handlers stacks
   ------------------------------
*/

#define SZ0                   0        /* Taille pile IT priorite 0 (octets) */
#define SZ1              3*1024        /* Taille pile IT priorite 1 (octets) */
#define SZ2                 512        /* Taille pile IT priorite 2 (octets) */
#define SZ3                 512        /* Taille pile IT priorite 3 (octets) */
#define SZ4                 512        /* Taille pile IT priorite 4 (octets) */
#define SZ5                 512        /* Taille pile IT priorite 5 (octets) */
#define SZ6                 512        /* Taille pile IT priorite 6 (octets) */
#define SZ7                 128        /* Taille pile IT priorite 7 (octets) */

#define OF0          (      SZ0)       /* Offset pour la base de la pile 0   */
#define OF1          (OF0 + SZ1)       /* Offset pour la base de la pile 1   */
#define OF2          (OF1 + SZ2)       /* Offset pour la base de la pile 2   */
#define OF3          (OF2 + SZ3)       /* Offset pour la base de la pile 3   */
#define OF4          (OF3 + SZ4)       /* Offset pour la base de la pile 4   */
#define OF5          (OF4 + SZ5)       /* Offset pour la base de la pile 5   */
#define OF6          (OF5 + SZ6)       /* Offset pour la base de la pile 6   */
#define OF7          (OF6 + SZ7)       /* Offset pour la base de la pile 7   */

#include <vmit.h>                      /* itstk endstk                       */
