/*****************************************************************************/
/*                                                                           */
/*  HyperPanel OS                       File "stm32m4/src/vm.s"              */
/*  Board - Assembly constants and definitions specific to stm32m4 board.    */
/*                                                                           */
/*  Author : JYA                        Last modification : 08/06/2023       */
/*                                                                           */
/*  (c) Copyright HyperPanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************
 * This macro is used to define a procedure written in assembly language.    *
 * STM32 targets only support Thumb ISA.                                     *
 *****************************************************************************/

.macro MACRO_PROCEDURE name            /* Procedure definition ------------- */
    .section    .text  , "ax"          /* Code section                       */
    .thumb_func                        /* Thumb function.                    */
    \name :                            /* Procedure entry point              */
.endm                                  /* ---------------------------------- */

.macro MACRO_ENDPROC                   /* End of procedure ----------------- */
.endm                                  /* ---------------------------------- */

.macro MACRO_LABEL name                /* Label definition ----------------- */
    .thumb_func                        /* Thumb function.                    */
    \name :                            /* Procedure entry point              */
.endm                                  /* ---------------------------------- */

/*****************************************************************************
 * Macro needed by "vmit.c" - raise and lower one software aised IRQ         *
 *****************************************************************************/

.macro GPIO_VMIT_SET                   /* Macro for GPIO PE5 aka GPIO_VMIT   */
    LDR        R0, =#0x40021018        /* Address of GPIO E BSR register     */
    LDR        R1, =#0x00000020        /* Bit 5.                             */
    STR        R1, [R0]                /* Set PIO high.                      */
.endm

.macro GPIO_VMIT_RESET                 /* Macro for GPIO PE5 aka GPIO_VMIT   */
    LDR        R2, =#0x40021018        /* Address of GPIOE E Set/Reset       */
    LDR        R3, =#0x00200000        /* Bit 5.                             */
    STR        R3, [R2]                /* Set PIO Low                        */
.endm

/*****************************************************************************
 * Macro needed by "tl_hrt2.s"                                               *
 * - Raise the GPIO4 PCM signal, GPIO4 is PE3                                *
 *   . GPIO E BSR register address is 0x40021018                             *
 *   . Bitmask is 1 << 3 = 8                                                 *
 * - Acknowledge the GPIO2 EXIT IRQ, GPIO2 is PE1 so EXTI IRQ is 1           *
 *   . EXTI PENDING register address is 40013C14                             *
 *   . Bitmask is 1 << 1 = 2                                                 *
 *****************************************************************************/

.macro GPIO_PCM_SET                    /* Macro for GPIO PE3 aka GPIO4       */
    LDR        R0, =#0x40021018        /* Address of GPIO E BSR register     */
    LDR        R1, =#0x00000008        /* Bit 3 is GPIO 3 set                */
    STR        R1, [R0]                /* Set PIO high.                      */
.endm

.macro EXTI_PCM_ACK_IRQ
    LDR        R0, =#0x40013C14        /* Address of EXTI PENDING register   */
    LDR        R1, =#0x00000002        /* Bit 1 is EXTI1                     */
    STR        R1, [R0]                /* Acknowledge interrupt EXT1         */
.endm

/*****************************************************************************
 * Macro needed by "int_cortex.s" - Sample the DWT counter and keep sample   *
 * in global "cycc_irq0"                                                     *
 *****************************************************************************/

.macro CYCC_IRQ0_GET                   /* Macro to sample DWR cycle counter  */
    LDR        R0, =cycc_irq0          /* Address of "cycc_irq0"             */
    LDR        R1, =#0xE0001004        /* Load the counter address           */
    LDR        R1 , [R1]               /* Sample the DWT current value       */
    STR        R1 , [R0]               /* Store sampled value to cntcyc_irq0 */
.endm

