/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/tsk_iolib.h            */
/*  Interface definitions for "tsk_iolib.c"                                  */
/*                                                                           */
/*  Author : JYA                        Last modification : 10/18/2022       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/

/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be included in space_appcont_xyz.c
*/

#ifndef VMK_IOLIB_H                    /* If this file is not already        */
#define VMK_IOLIB_H                    /* included, include it.              */


/*
    2    Structures and typedefs
    ____________________________


   2.1  One entry of the "mod_def[]" table
   ---------------------------------------
*/

typedef struct                         /* Files req/ecr/lec/aut et compteur  */
  BEGIN                                /* ---------------------------------- */
    CHAR           *modname          ; /* Module name address                */
    UINT           flags             ; /* "drv_init_driver" flags parameter  */
    INT            nbdev             ; /* Count of used devices              */
    INT            rtemincode        ; /* Add_uroute min code event          */
    INT            rtemaxcode        ; /* Add_uroute max code event          */
  END              Mod_def           ; /* ---------------------------------- */
 
/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/

IWORD io_list_drivers     (char*,int,int*                                    );
IWORD io_list_devices     (const char*,char*,int,int*                        );
IWORD io_list_tags        (const char*,char*,int,int*                        );
IWORD io_tag_number       (const char*,const char*,int*                      );
IWORD io_tag_name         (const char*,int, char*, int                       );

IWORD io_open             (const char*,const char*,const char*,int*          );
IWORD io_close            (int                                               );
IWORD io_alloc_sub        (int,const char*,int*                              );
IWORD io_free_sub         (int                                               );

IWORD io_write            (int,unsigned char*,int,int                        );
IWORD io_send             (int,unsigned char*,int,int*                       );
IWORD io_read             (int,unsigned char*,int,int*                       );
IWORD io_receive          (int,int,int,int*                                  );

IWORD io_setval           (int,const char*                                   );
IWORD io_getval           (int,char*,int                                     );

IWORD io_bufsize          (int,int,int*,int*                                 );

#endif                                 /* End of VMK_IOLIB_H                 */

#ifdef SPACE_APPCONT_XYZ_DATA_C

INT            iolib_modd[NB_MODD]   ; /* MODD values storage                */
UINT           iolib_rteid[NB_MODD]  ;
UCHAR          iolib_iodnb[NB_MODD]  ;
INT            iolib_iod  [NB_IOD]   ;

const INT      iolib_nbmodd = NB_MODD; /* Size of the "iolib_modd[]" array   */
const INT      iolib_nbiod  = NB_IOD ; /* Size of the "iolib_iod[]" array    */

#endif                                 /* End of SPACE_APPCONT_XYZ_C         */


