/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS -                     File "os/vmk/lib/tsk_netlib.c"       */
/*  ...                                                                      */
/*                                                                           */
/*  Author : ...                        Last modification : 04/19/2022       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================



    1    Purpose of this module
    ___________________________


         Bla Bla
   
    Some rules for on-line documentation generation :

    #PRIVATE   ... #END     Not generated in on-line documentation.
    #DIAGRAM   ... #END     Diagram, drawing, schematic.
    #CODE      ... #END     Source code example (ex: typedef).
    #ARRAY     ... #END     Array (ex: explanation following typedef).
    #INDEX     ... #END     Sub-chapters list, procedure list.
    #PCODE     ... #END     Pseudo-code.
    #PROCDESCR ... #END     Procedure interface description.

#PRIVATE ----------------------------------------------------------------------

NOTES ========================================================================

CODE =========================================================================

DOC ==========================================================================

TESTS ========================================================================

EVOLUTIONS ===================================================================


#END --------------------------------------------------------------------------



    2    User manual
    ________________


         2.1  xxxxxxx
              2.1.1  xxxxxx

    2.1  xxxxxxx
         Bla bla

    3    Porting guide
    __________________


         3.1  xxxxxxx
              3.1.1  xxxxxx

    3.1  xxxxxxx
         Bla bla


    4    Include files and external references
    __________________________________________
*/


#include <stdtyp.l>                    /* Hyperpanel definitions & constants */
#include <event.l>                     /* RESP_WRITE                         */
#include <aut.h>                       /* S_evt                              */
#include <vmk.h>                       /* "vmk.c" prototypes and constants   */
#include <drv.h>                       /* "drv.c" prototypes and constants   */
#include <text.h>                      /* hsprintf                           */
#include <mem_buf.h>                   /* alloc_buf free_buf                 */

#include <tsk_iolib.h>                 /* Mod_def                            */
#include <tsk_netlib.h>                /* Net_iod                            */
                                       /*                                    */
#include <systemm.txt>                 /* WAIT_CODERES                       */
#include <messagem.txt>                /* BAD_IOD                            */

#ifdef TEST                            /* For unitary tests.                 */
#endif                                 /* End of ifdef TEST.                 */

/* External reference from space_appcont_xyz.c" .............................*/

extern const Mod_def netlib_moddef[]; /* Table of requested modules          */
                                      /*                                     */
extern INT           netlib_modd [] ; /* Storage for the MODD's              */
extern UINT          netlib_rteid[] ; /* Storage for route identifiers       */
extern UCHAR         netlib_iodnb[] ; /* Count of allocated devices          */
extern Net_iod       netlib_iod  [] ; /* Storage for the IOD's               */
                                      /*                                     */
extern const INT     netlib_nbmodd  ; /* Size netlib_moddef[] netlib_modd[]  */
extern const INT     netlib_nbiod   ; /* Size of "netlib_iod[]"              */


extern const Sys_driver * const vmos_driver[]; /* Table des desc. drivers    */
extern const USHORT  vmos_nbdriver   ; /* Nombre d'elements de vmos_driver   */

/* External reference from ptk_lib.c ........................................*/

extern IWORD         vlgv            ; /* Maximum string size in bytes       */

/* External reference from the applicative software .........................*/

extern UINT          myapp_memuid    ; /* Memry user identifier              */

/* External reference from ..................................................*/


/*
    5    Inner workings of the module
    _________________________________


         5.1  xxxxx
              5.1.1  xxxxx

         5.2  xxxxx
              5.2.1  xxxx


    6    Exported (public) variables
    ________________________________
*/

/*
    7    Internal constants of the module
    _____________________________________
*/

/*
    8    Internal data types of the module
    ______________________________________
*/

static IWORD net_get_iodidx  (INT*,Net_iod**                                 );
static IWORD net_chk_driver  (const CHAR*,INT*,INT*                          );
static IWORD net_chk_iod     (INT,INT,INT*,INT*                              );

/*
    9    Internal (private) variables of the module
    _______________________________________________
*/

/*
    10   Beginning of the code
    __________________________

------------------ Network API functions --------------------------------------

net_open_device    Opens a network interface device
net_close_device   Closes a network interface device
net_alloc_sub      Allocates one sub-channel of a network interface devive
net_free_sub       Free one sub_channel

net_bufsize        Get send/receive data buffer length
net_write          Blocking send
net_send           Non blocking send
net_read           Blocking read
net_receive        Non blocking receive

net_setval         Set parameter values
net_getval         Read parameter values

net_nodeid_req     Sends a request to allocate a node identifier
net_nodeid_resp    Sends a response holding an allocated node identifier
net_nodename_req   Sends a request to get a node name / serial number
net_nodename_resp  Sends a response holding a node name / serial number
net_objectid_req   Sends a request to allocate an object identifier
net_objectid_resp  Sends a response holding an allocated object identifier
net_send_cmd       Sends a command
net_send_val       Sends a value

------------------ Internal subroutines ---------------------------------------
net_get_ioidx      Get a free entry in the "netlib_iod[]" table
net_chk_driver     Check a network interface driver name
net_chk_iod        Check a network IOD
net_chk_nodeid     Check a network node identifier


*/


/*  Procedure net_open_device -----------------------------------------------*/
/*
    Purpose : Open a network interface driver

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_open_device( modbanem, devname, taglist, iod )

    Input     : modname  CHAR*      Driver module name
                devname  CHAR*      Device name
                taglist  CHAR*      List of tags for "drv_open_device"

    Output    : iod      INT*       IO channel identifier

    Input     : x        IWORD      C

...............................................................................
*/

IWORD  PROCEDURE net_open_device(const char *modname, const char *devname,
                                 const char *taglist,int *iod            )

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    IWORD          ioidx             ; /* "netlib_iod[]" index               */
    IWORD          moidx             ; /* Loop counter/netlib_modd[] index   */
    const Mod_def  *pmd              ; /* Pointer in "netlib_moddef[]" table */
    CHAR           name[32]          ; /* For "drv_alloc_device"             */
    Net_iod        *net_iod          ; /* "netlib_iod[ioidx]" address        */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_open_device")       ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Check that we have at least one available entry in the          *
 * -------   "netlib_iod[]" IOD storage table. We call the "net_get_iodidx"  *
 *           subroutine which returns the first "ioidx" free entry index.    *
 *           If the table is full, function return value is NOT_ENOUGH_MEM   *
 *****************************************************************************/

    r = net_get_iodidx ( &ioidx  ,     /* Get the first free entry index     */
                         &net_iod )  ; /* in "netlib_iod[]"                  */
    if ( r )                           /* if we get NOT_ENOUGH_MEM, we exit  */
      goto exit                      ; /* with that error code value         */


/*****************************************************************************
 * Step  2 : Search for the "modname" name in the "modlist" const table      *
 * -------   and exit with BAD_NOMP if this name is not found in the const   *
 *           "netlib_moddef[]" table. "moidx" is an index in                 *
 *           "netlib_moddef[]" and ""pmd" is "iolib_moddef[moidx]" address   *
 *           --------------------------------------------------------------- *
 *           We remind that netlib_moddef" table is an array of "Mod_def".   *
 *           This const array should be defined in "space_appcont_xyz.c".    *
 *           We also remind that "space_appcont.xyz.c" includes              *
 *           "tsk_netlib.h" in which a "const INT" variable named            *
 *           "netlib_nbmodd" is defined, its value beeing the                *
 *           "netlib_moddef[]" table size.                                   *
 *****************************************************************************/

    r = BAD_NOMP                     ; /* Let's assume we will fail          */
                                       /*                                    */
    for (moidx=0, pmd=netlib_moddef ;  /* We loop on "netlib_nbmodd" entries */
         moidx LT netlib_nbmodd     ;  /* of the "netlib_moddef[]" table,    */
         moidx++, pmd++              ) /* "moidx" is the table index whilst  */
      BEGIN                            /* "pmd" is "&iolib_moddef[moidx]"    */
        if(Strcmp(modname,             /* We search for the "modname" input  */
                  pmd->modname) EQ 0)  /* parameter string, if we find it    */
          goto step3                 ; /* we are so happy, we jump to 2 !!   */
      END_FOR                          /* Not found, continue looping        */
                                       /*                                    */
    goto exit                        ; /* We are so sad, exit with BAD_NOMP  */


/*****************************************************************************
 * Step 3 : If the driver is not yet initialized, if the "netlib_modd[moidx]"*
 * ------   global variable value is now 0, then we have to send a REQ_INIT  *
 *          initialization request event to the VMIO driver FSM. To do this, *
 *          we call the "drv_init_driver" procedure. The VMIO driver FSM     *
 *          responds with a RESP_INIT event, we here unschedule until        *
 *          reception of this of this response event                         *
 *****************************************************************************/

    step3                            : /* Driver initialization if needed    */
                                       /*                                    */
    if( netlib_modd[moidx] )           /* If the module is already           */
      goto step4                     ; /* initialized, skip step3 and step4  */
                                       /*                                    */
    r = drv_init_driver(               /* Sends a REQ_INIT                   */
                 (CHAR*)modname     ,  /* Name of the driver module          */
                 0                  ,  /* Node number = local (0)            */
                 1                  ,  /* Reqid                              */
                 netlib_modd + moidx); /* Module identifier MODD             */
                                       /*                                    */
    if ( r ) goto exit               ; /* Exit if error                      */


/*****************************************************************************
 * Step 4 : The VMIO driver is ready. We are going now to allocate the       *
 * ------   device. We call the "drv_alloc_device" procedure, this procedure *
 *          sends a REQ_ALLOC event to the VMIO driver FSM. This automaton   *
 *          will then send us a RESP_ALLOC response event. Here we           *
 *          unschedule until this event is received. We keep the returned    *
 *          IOD value in "netlib_iod[ioidx]" and we increment the count      *
 *          "iolib_iodnb[moidx]" of allocated devices for the driver         *
 *          ---------------------------------------------------------------- *
 *          We remind that bits b8..15 of any device IOD is the device       *
 *          number, starting with 0. So here, if there is no error, we get   *
 *          "device" from bits b8..15 of "iod"                               *
 *****************************************************************************/

    step4                            : /* Allocate the device                */
                                       /*                                    */
    hsprintf(name,"%s\\%s",            /* The device name is                 */
             modname      ,            /* "DRIVER_NAME\DEV" whereas "dev"    */
             devname        )        ; /* is the device name                 */
                                       /*                                    */
    r = drv_alloc_device(              /* Allocate device "name"             */
                 netlib_modd[moidx],   /* Module Descriptor Id               */
                 name              ,   /* Device name = name                 */
                 ""                ,   /* Tagini  = empty                    */
                 ""                ,   /* Taglist = empty                    */
                 pmd->flags        ,   /* Flags   = 0 or NBLOCKING_IO        */
                 2                 ,   /* Reqid                              */
                 0                 ,   /* Route ID   = empty                 */
                 0                 ,   /* Route ID 2 = empty                 */
                 iod               ) ; /* Returned I/O descriptor            */
                                       /*                                    */
    if ( r ) goto exit               ; /* Exit if error                      */
                                       /*                                    */
    net_iod = netlib_iod + ioidx     ; /* Address of allocated entry         */
                                       /*                                    */
    net_iod->iod       = *iod        ; /* Keep the allocated IOD             */
    net_iod->nodeid    = 0xFFFF      ; /* Set with an empty value            */
                                       /*                                    */
    netlib_iodnb[moidx] ++           ; /* Update count of allocated devices  */


/*****************************************************************************
 * Step 5 : Here we initialize the hardware of the device. We call the       *
 * ------   "device_open_device" procedure. This procedures sends a REQ_OPEN *
 *          request event to the VMIO driver automaton, that is located      *
 *          inside the IO container. Then we ask to be unscheduled until the *
 *          AUT_LED send us the RESP_OPEN response event                     *
 *****************************************************************************/

    r = drv_open_device(               /* Sends a REQ_OPEN                   */
                    *iod          ,    /* I/O descriptor                     */
                    (CHAR*)taglist,    /* Taglist                            */
                    3              ) ; /* Reqid                              */


/*****************************************************************************
 * Step 6 : Procedure exit. If there is any error, we return we call the     *
 * ------   "vhalt" OS halt procedure.                                       *
 *****************************************************************************/

    exit                             : /* Exit label                         */
                                       /*                                    */
    if ( r )                           /* If we have any error, we           */
      vhalt(0)                       ; /* call the OS halt function          */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_open_device"        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_close_device ----------------------------------------------*/
/*
    Purpose : Closes a network interface device, which must have been opened
              using net_open_device. If all devices of the driver are closed,
              terminates the driver

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_close_device( iod )

    Input     : iod      INT        Opened device IO descriptor

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_close_device(int iod)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */
    INT            maj               ; /* Major number << 16                 */
    Net_iod       *p                 ; /* Address of "netlib_iod[ioidx]      */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_close_device")      ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device " procedure. We do not allow a call to the      *
 *          present procedure with an IOD which has not been allocated using *
 *          "net_open_device". We call the "net_chk_iod" subroutine          *
 *****************************************************************************/

    r = net_chk_iod(iod,0,             /* Check that "iod" is one of         */
                    &moidx,&ioidx)   ; /* "net_open_device" returned value   */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


/*****************************************************************************
 * Step  2 : Search for the corresponding MODD in the "netlib_modd[]" table  *
 * -------   and set "moidx" to the index the MODD has been found. We simply *
 *           do a simple relational filtering because bits b16..23 of IOD's  *
 *           and MODD's are the major number, so:                            *
 *           - We set "maj" to bitsb16..23 of "iod", we extract the major    *
 *             number from the IOD.                                          *
 *           - Then we loop on entries of "netlib_modd[]" table of MODD's,   *
 *             using "moidx" as the index, and we compare bits b 16..23 of   *
 *             "iolib_modd[moidx]" with "maj". We stop looping when we       *
 *             have found our MODD, we jump to next step 3                   *
 *           --------------------------------------------------------------- *
 *           We remind that with drivers, we have at mod one MODD for every  *
 *           driver and one distinct major number per driver. As a           *
 *           consequence, the "netlib_modd[]" table cannot hold more than    *
 *           one MODD for a given major number                               *
 *           --------------------------------------------------------------- *
 *           Unless we have incoherent data (a bug) the search should never  *
 *           fail, we always have one MODD (and only one) for every IOD      *
 *****************************************************************************/

    r   = BAD_IOD                    ; /* Let'a assume we will fail          */
    maj = iod AND 0x00FF0000         ; /* Get the major number field value   */
                                       /*                                    */
    for(moidx = 0              ;       /* We loop with "moidx", searching    */
        moidx LT netlib_nbmodd ;       /* the "netdlib_modd[]" table for the */
        moidx++                 )      /* MODD which holds in bits b16..23   */
      BEGIN                            /* the major number for "iod".        */
        if ( (netlib_modd[moidx] AND   /* If this MODD holds the "maj" major */
              0x00FF0000) EQ maj    )  /* number, we are happy, we stop      */
          goto step3                 ; /* looping and jump to next step 3    */
      END_FOR                          /*                                    */
                                       /* The search failed: we have a bug   */
    goto exit                        ; /* since this should never happens    */

/*****************************************************************************
 * Step 3 : Close the device. We have to call "drv_close_device" for "iod".  *
 * -------  This procedure sends a REQ_CLOSE request event to the driver     *
 *          VMIO automaton and here we unschedule until the corresponding    *
 *          RESP_CLOSE, sent by AUT_xyz, is received                         *
 *****************************************************************************/

    step3                            : /* Close the device                   */
                                       /*                                    */
    r = drv_close_device(iod         , /* IOD of the device to be closed     */
                         4        )  ; /* Reqid                              */
    if ( r )                           /* If we have any error, we exit      */
      goto exit                      ; /* with that error code value         */


/*****************************************************************************
 * Step 4 : Free the device. We have to send to the AUT_xyz VMIO automaton a *
 * ------   REQ_FREE request event. We have to call the "drv_free" procedure *
 *          to do that. Then, we unschedule until the RESP_FREE response     *
 *          event is received.                                               *
 *****************************************************************************/

    r = drv_free(iod,                  /* IOD of the device to be freed      */
                 5   )               ; /* reqid                              */
    if ( r )                           /* This call should never fail        */
      goto exit                      ; /*                                    */


/*****************************************************************************
 * Step 5 :  We free the "ioidx" entry of the "netlib_iod[]" table and then  *
 * -------   decrement the count of allocated devices for that module, the   *
 *           "netlib_iodnb[moidx]" global variable. If this count is still   *
 *           strictly positive, if there is one or more IOD's, we are done   *
 *           and we exirt with "r=0" else we continue with next step 6.      *
 *****************************************************************************/

    p         = netlib_iod + ioidx   ; /* Address of "netlib_iod[ioidx]"     */
    p->iod    = 0                    ; /* Clear the IOD value                */
    p->nodeid = 0xFFFF               ; /* Empty node identifier              */
                                       /*                                    */
    netlib_iodnb[moidx] --           ; /* Update the count of allocated      */
                                       /* devices/IOD's for the driver       */
                                       /*                                    */
    if (netlib_iodnb[moidx] GT 0)      /* If we still have one allocated     */
      goto exit                      ; /* device, we are done                */


/*****************************************************************************
 * Step 6 : The count of remaining allocated devices has reached 0, so       *
 * ------   we terminate the driver, we call the "drv_end" interface         *
 *          procedure that sends a REQ_END request event to the driver's     *
 *          VMIO  autiomaton and unschedule us until the RESP_END response   *
 *          event is received.                                               *
 *****************************************************************************/

    r = drv_end(netlib_modd[moidx] ,    /* We terminate the use of this      */
                6                  )  ; /* driver, we call "drv_end"         */

    exit                             : /* Exit label                         */
    if ( r )                           /* if we have any error, we           */
      vhalt(0)                       ; /* halt.                              */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_close_device")      ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_alloc_sub -------------------------------------------------*/
/*
    Purpose : Allocates one sub-channel of one opened network interface
              device - Typically one receive filter.

    Author : JYA                                 Last modification : 11/13/2022

...............................................................................

    Calling example : net_alloc_sub( iod, tagini, iodsub )

    Input     : iod      INT        Opened device IO descriptor   
                tagini   INT        Sub-channel selector tag list

    Output    : iodsub   INT*       Allocated sub-channel IO descriptor

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_alloc_sub(int iod, const char *tagini, int *iodsub)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Index in "netlib_modd[]"           */
    INT            ioidx             ; /* Index in "netlib_iod[]"            */
    INT            iodsubidx         ; /* Index in "netlib_iod[]"            */
    Net_iod        *pnet             ; /* Pointer to "netlib_iod[ioidx]      */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_alloc_sub")         ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, that it is a value          *
 * ------   previously returned by the "net_open" procedure. We do not allow *
 *          a call to the present procedure with an IOD which hs not been    *
 *          allocated using "net_open". We call the "net_chk_iod" subroutine *
 *          and we check that "iod" is a device IOD and not a sub-channel IOD*
 *****************************************************************************/

    r = net_chk_iod(iod,0,             /* Check that "iod" is one device     */
                    &moidx,&ioidx)   ; /* IOD from "net_open_device"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */

/*****************************************************************************
 * Step  2 : Check we have at least one free entry in the "netlib_iod[]"     *
 * -------   table, because we will need to store the "*iodsub" IOD value.   *
 *           We use the "net_get_iodidx" subroutine, which returns           *
 *           NOT_ENOUGH_MEM if table is full.                                *
 *****************************************************************************/

    r = net_get_iodidx( &iodsubidx ,   /* Check if "netlib_iod[]" is full,   */
                        &pnet       ); /* and return first free index        */
    if ( r )                           /* If the table is full, we exit      */
      goto exit                      ; /* with NOT_ENOUGH_MEM                */

/*****************************************************************************
 * Step 3 : We are going to allocate one subchannel. We call the             *
 * -------  "drv_alloc_subchan" procedure, this procedure sends a            *
 *          REQ_ALLOC_SUB event to the driver's VMIO automaton. This         *
 *          automaton will then send us a RESP_ALLOC_SUB response event.     *
 *          Here we unschedule (in "drv_alloc_subchan") until this event is  *
 *          received.                                                        *
 *****************************************************************************/

    r = drv_alloc_subchan(             /* Send a REQ_ALLOC_SUB               */
                     iod           ,   /* I/O descriptor of controller       */
                     (CHAR*)tagini ,   /* Tagini  = Filter selector          */
                     ""            ,   /* Taglist = empty                    */
                     0             ,   /* Flags                              */
                     7             ,   /* Reqid                              */
                     0             ,   /* Route ID   = empty                 */
                     0             ,   /* Route ID 2 = empty                 */
                     iodsub        ) ; /* Returned I/O descriptor            */
                                       /*                                    */
    if ( r )                           /*                                    */
      goto exit                      ; /*                                    */

/*****************************************************************************
 * Step 4 : We are successful, keep the "*iodsub" IOD value in the           *
 * ------   "iolib_iod[iodsubidx]"                                           *
 *****************************************************************************/

    pnet->iod = *iodsub              ; /* Keep the allocated sub-channel IOD */
                                       /* at index "iodsubidx" of "iolib_iod"*/


/*****************************************************************************
 * Step 5 : If we have any error, we call the OS halt "vhalt" function       *
 * ------                                                                    *
 *****************************************************************************/

    exit                             :

    if ( r )                           /* If we have any error, we call      */
      vhalt(0)                       ; /* the OS halt function               */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_alloc_sub")         ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure net_free_sub --------------------------------------------------*/
/*
    Purpose : Free one sub-channel of one device. The sub-channel must have
              been allocated using "net_alloc_sub"

    Author : JYA                                 Last modification : 11/13/2022

...............................................................................

    Calling example : net_free_sub ( iodsub )

    Input     : iodsub   int        Sub-channel IO descriptor

    Output    : valproc  IWORD      Function return/error code

...............................................................................
*/

IWORD  PROCEDURE net_free_sub(int iodsub)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_free_sub")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, that it is a value          *
 * ------   previously returned by the "net_alloc_sub" procedure. We call    *
 *          the "net_chk_iod" procedure that checjs that "iodsub" is a       *
 *          sub-channel IOD and not a device IOD.                            *
 *****************************************************************************/

    r = net_chk_iod(iodsub , 1,        /* Check that "iod" is one            */
                    &moidx,&ioidx)   ; /* "drv_alloc_device" returned value  */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


/*****************************************************************************
 * Step 2 : Free the subchannel. We call "drv_free_subchan", this procedure  *
 * ------   sends a REQ_FREE_SUB request event to the driver's VMIO automaton*
 *          and here we unschedule until the corresponding RESP_FREE_SUB,    *
 *          sent by the driver's VMIO FSM is received.                       *
 *****************************************************************************/

    r = drv_free_subchan(iodsub ,      /* IOD of the subchannel to be freed  */
                         8       )   ; /* Reqid                              */
    if ( r )                           /* If we have any error, we exit      */
      goto exit                      ; /* with that error code value         */


/*****************************************************************************
 * Step 3 : Fre the "ioidx" entry of table "iolib_iod[]". We wimply need     *
 * -------  to clear that value                                              *
 *****************************************************************************/

    netlib_iod[ioidx].iod = 0        ; /* Clear/free entry "ioidx"           */


/*****************************************************************************
 * Step 4 : If we have any error, we call the OS halt "vhalt" function       *
 * ------                                                                    *
 *****************************************************************************/
    exit                             : /* Procedure exit label               */
                                       /*                                    */
    if ( r )                           /* If we have any error, we call      */
      vhalt(0)                       ; /* the OS halt function               */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_free_sub")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_bufsize ---------------------------------------------------*/
/*
    Purpose : Returns the "len" neede buffer size in bytes in order to
              accomodate for "lg" data bytes.

    Author : JYA                                 Last modification : 10/19/2022

...............................................................................

    Calling example : 

    Input     : x        IWORD      C

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_bufsize(int iod, int lg, int *lgmax, int *offset, int *len)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */
    INT            trail             ; /* Unused value                       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_bufsize")           ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, that it is previously       *
 * ------   returned by the "net_open_device" or "net_open_protocol". We do  *
 *          not allow  a call to the present procedure with an IOD which has *
 *          not been allocated using "io_open_device/protocol". We call the  *
 *         "net_chk_iod" subroutine                                          *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one            */
                    &moidx,&ioidx)   ; /* "net_open_device" returned value   */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


/*****************************************************************************
 * Step  2 : Call the "drv_get_bufsize" function for the "write" case        *
 * --------                                                                  *
 *****************************************************************************/

    r = drv_get_bufsize( iod    ,      /* Unique identifier of the channel   */
                         lg     ,      /* Count of user data bytes           */
                         0      ,      /* 0:write 1:read                     */
                         lgmax  ,      /* Maximum count of user data bytes   */
                         offset ,      /* Offset to the 1stt byte to send    */
                         &trail ,      /* Trailer size in bytes              */
                         len     )   ; /* Equal to "offset + lg + trail      */


    exit                             : /* Exit label                         */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_bufsize")           ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_write -----------------------------------------------------*/
/*
    Purpose : Writes one data buffer. This is a blocking call, caller is
              unscheduled until the end of sending/write, despite the fact
              the driver has been opened in NBLOCKING_IO

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : 

    Input     : iod      INT        C

    Output    : 

    Notes : - The user data always starts at "buf", no offset is used since
              if the driver or protocol needs one, then the present procedure
              allocates a temporary buffer and copy the user data into it.

            - With CAN, a 8 byte message does not needs a one byte header, a
              more than 8 bytes needs a header (see PDO and SDO). Distinct
              function codes may be useed, implying the use of "flags" for the
              "drv_write" procedure (TDB)

...............................................................................
*/

IWORD  PROCEDURE net_write(int iod, unsigned char *buf, int lg,int flags)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in netlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "netdlib_iod[]"           */
    INT            lgmax             ; /* Max data bytes count               */
    INT            offset, trail, len; /* Returned by "drv_get_buf_size"     */
    UINT           waitlist[1][3]    ; /* Parameter of "waitevt_task         */
    UCHAR          *tmp              ; /* Temporary buffer address           */
    UCHAR          *tmpdata          ; /* User data address in "tmp"         */
    UCHAR          *bufrd            ; /* Read pointer into "buf"            */
    IWORD          chunkcnt          ; /* Chunk counter                      */
    IWORD          chunkmax          ; /* Count of needed chuncks            */
    IWORD          chunkusr          ; /* Count of user bytes per chunk      */
    IWORD          chunksz           ; /* Current chunk size in bytes        */
    IWORD          lgrem             ; /* Count of remaining bytes           */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_write")             ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, that is previously returned *
 * ------   by "net_open_device" or "net_open_protocol" procedure. We do not *
 *          allow  a call to the present procedure with an IOD which has not *
 *          been  allocated using "net_open_device/protocol". We call the    *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one from       */
                    &moidx,&ioidx)   ; /* "net_open_device/protocol"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


/*****************************************************************************
 * Step 2 : With a protocol, a header in the sending buffer is needed        *
 * ------   The "drv_get_bufsize" procedure returns the offset where the     *
 *          first user data byte should be located. We assume that the       *
 *          caller of "io_write" did the thing right, the he took in account *
 *          this same offset correctly, wrote his first data byte at "offset"*
 *          The "drv_write" procedure checks that the "offset" it gets has   *
 *          exactly the correct value otherwise BAD_PARAMETER is returned    *
 *****************************************************************************/

    r = drv_get_bufsize( iod     ,     /* iod    IO descriptor               */
                         lg + 1  ,     /* lg     Count of user bytes         */
                         0       ,     /* wrrd   0:write 1:read              */
                         &lgmax  ,     /* lgmax  Max data byte count         */
                         &offset ,     /* offset Offset to 1st byte to send  */
                         &trail  ,     /* trail  Trailer sier in bytes       */
                         &len      ) ; /* len    "offset + lg + trail"       */
    if ( r )                           /* If we fail, we exit, returning the */
      goto exit                      ; /* "drv_get_bufsize" error code value */
                                       /*                                    */

/*****************************************************************************
 * Step 3 : Initialization of local variables:                               *
 * ------   - This procedure allocates a temporary buffer and will keep its  *
 *            address is the local "tmp", that we set to HNULL in case the   *
 *            call to "drv_get_buffsize" wouls fail.                         *
 *          - We prepare a simple waiting list to be used with               *
 *            "waitevt_task", we will need to wait for a single RESP_WRITE   *
 *            for "iod", so code is WAIT_CODERES and parameters are the      *
 *            RESP_WRITE code value and the "iod" "reserve" field value      *
 *          - We compute the "chunkmax" count of needed chunks in order to   *
 *            send the "lg" user data bytes. Each PDU can carry a maximum of *
 *            "lgmax" bytes but in our case we needed to add one byte for    *
 *            the chunk number, so each chunk will carry "lgmax-1" user      *
 *            bytes so "chunkmax" is (lg + (lgmax -1) -1) / (lgmax - 1)      *
 *          - We will use a "pt" read pointer into "buf", we initialize it   *
 *            to the "buf" user data buffer address. We also initialize the  *
 *            "chunkcnt" current chunk number to 0 and and the "lgrem" count *
 *            of remaining user data bytes to be sent to "lg"                *
 *****************************************************************************/

    tmp            = HNULL           ; /* Let's assume we will fail          */
                                       /*                                    */
    waitlist[0][0] = WAIT_CODERES    ; /* Test "code" and "reserve" fields   */
    waitlist[0][1] = RESP_WRITE      ; /* The awaited value for "code"       */
    waitlist[0][2] = iod             ; /* The awaited value for "reserve"    */
                                       /*                                    */
    chunkusr    = lgmax -1           ; /* Count of user data bytes per chunk */
                                       /*                                    */
    chunkmax    = (lg + chunkusr -1)   /* (lg + (lgmax -1) -1)               */
                /   chunkusr         ; /* / (lgmax - 1)                      */
                                       /*                                    */
    bufrd       = buf                ; /* Initialize "bufrd" read pointer    */
    chunkcnt    = 0                  ; /* Initialize the chunk counter       */
    lgrem       = lg                 ; /* Count of remaining bytes to send   */

    if (chunkmax GT 16)
      BEGIN
        r = -1                       ;
        goto exit                    ;
      END_IF

/*****************************************************************************
 * Step 4 : Always allocate a temporary buffer because:
 * -------  - If the user provided data exceeds the protocol maximum PDU     *
 *            size, we will do a a loop, we will fragment the user data. We  *
 *            need a "small" temporary buffer where we will copy data chunks *
 *          - If the protocol needs a header area, we do not want to burden  *
 *            the user with that offset and therrfore we allocate.           *
 *          - We always add one byte befor the user data which is a chunk    *
 *            counter and the chunk count.                                   *
 *****************************************************************************/

    r = alloc_buf(myapp_memuid ,       /* Memory user id                     */
                  len          ,       /* Size in bytes of requested buffer  */
                  0            ,       /* Flags                              */
                  &tmp           )   ; /* Address of allocated buffer        */
    if ( r )                           /* If buffer allocation fails, we     */
      goto exit                      ; /* exit with the "alloc_buf" error    */
                                       /*                                    */
    tmpdata = tmp + offset           ; /* First data byte will be put just   */
                                       /* after the network header           */

/*****************************************************************************
 * Step  5 : Fill the "tmp" chunk buffer:                                    *
 * --------  - We copy the header structure, in other words the network      *
 *             address an associated flags. This structure is located at     *
 *             the beginning of "buf" and we copy it to the beginning of     *
 *             "tmp". Its size is "offset" bytes.                            *
 *           - First data byte holds the current chunck number (0..15) in    *
 *             bits 4..7 and the chunk total count in bits b0..3. We put     *
 *             this byte at address "tmpdata"                                *
 *             count (0->1, 1->2, ... 15->16). We put is at "tmpdata".       *
 *           - Then we copy user data bytes, that we copy from "bufrd" to    *
 *             "tmpdata + 1". The "chunksz" number of copied bytes is        *
 *             "chunkusr" but for the last chunk which may be smaller, only  *
 *             "lgrem" bytes.                                                *
 *           --------------------------------------------------------------- *
 *           It has to be noted that the driver (drv_write) only takes in    *
 *           account the network header contents if NET_ADR bit is set in    *
 *           the "flags" field                                               *
 *****************************************************************************/

     step5                           : /* Chunks sending loop beginning      */
                                       /*                                    */
     Memcpy(tmp        ,               /* The header is "offset" bytes that  */
            buf        ,               /* we copy from the beginning of      */
            offset      )            ; /* "buf" to the beginning of "tmp"    */
                                       /*                                    */
     tmpdata[0]  = (chunkcnt << 4)     /* Encode in the first byte the       */
                 | (chunkmax - 1 )   ; /* current chunk number and the total */
                                       /* count of chunks (minus 1)          */
     chunksz = lgrem GE chunkusr ?     /* The current chunk size is the      */
               chunkusr          :     /* "chunkusr" byte count but for the  */
               lgrem                 ; /* last one, which is "lgrem"         */
                                       /*                                    */
     Memcpy(tmpdata + 1 ,              /* Copy "chunksz" user data bytes     */
            bufrd       ,              /* from "bufrd" to "tmpdata + 1"      */
            chunksz      )           ; /*                                    */


/*****************************************************************************
 * Step 6 : Send using "iod" the "lg" bytes that are in the "snd" data       *
 * ------   buffer.  We call the "drv_write" procedure that sends a          *
 *          REQ_WRITE request event to the driver's VMIO automaton.          *
 *****************************************************************************/

    r = drv_write(iod         ,        /* IOD of serial controller DEV0      */
                  tmp         ,        /* Buffer Address                     */
                  offset      ,        /* Buffer Offset                      */
                  1 + chunksz ,        /* Number of bytes to be sent         */
                  flags       ,        /* Flags                              */
                  12           )     ; /* Reqid                              */
    if ( r )                           /* If we fail, we exit, returning the */
      goto exit                      ; /* "drv_write" error code value       */
                                       /*                                    */

/*****************************************************************************
 * Step 7 : Wait for the RESP_WRITE driver or protocol response event. We    *
 * -------- call "waitevt_task" with a one item wait list.                   *
 *****************************************************************************/

    r = waitevt_task(waitlist ,        /* Address of waiting list            */
                     1        ,        /* Size of "waitlist[]"               */
                     0        ,        /* No maximum waiting time            */
                     0          )    ; /* Do not purge previous events       */
                                       /*                                    */
    if ( r )                           /* If we fail, we exit, returning the */
      goto exit                      ; /* "waitevt_task" error code value    */



/*****************************************************************************
 * Step 8 : Update counters an pointers and if we are not done, loop again:  *
 * -------  - Decrement the "lgrem" count of remaining user bytes to be sent *
 *            with the "chunksz" count of just sent bytes.                   *
 *          - Increment the "bufrd" read pointer with "chunksz"              *
 *          - Increment by 1 the "chunkcnt" current chunk number             *
 *          Then if "lgrem" is not 0, do another loop step, jump to "step5"  *
 *****************************************************************************/

    lgrem -= chunksz                 ; /* Count of remaining user data bytes */
    bufrd += chunksz                 ; /* Read address in "buf"              */
    chunkcnt ++                      ; /* Current chunk number               */
                                       /*                                    */
    if ( lgrem )                       /* If we have remaining bytes, do     */
      goto step5                     ; /* another loop step                  */

    exit                             : /* Exit label                         */
    if ( tmp )
      free_buf( tmp, 0 )             ;

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_write")             ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_send ------------------------------------------------------*/
/*
    Purpose : Request the sending of one data buffer to the network interface
              device driver or with a protocol. This function is non blocking.

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_send( iod, buf, lg )

    Input     : x        IWORD      C

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_send(int iod, unsigned char *buf, int lg,int* count)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */
/*  INT            lgmax             ; /  Max data bytes count               */
/*  INT            offset, trail, len; /  Returned by "drv_get_buf_size"     */
/*  UINT           waitlist[1][3]    ; /  Parameter of "waitevt_task         */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_send")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device" or "net_open_protocol". We do not allow a call *
 *          to the present procedure with an IOD which has not been          *
 *          allocated using "net_open_device/protocol". We call the          *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one of         */
                    &moidx,&ioidx)   ; /* "net_open_device/protocol"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */



/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_send")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_read ------------------------------------------------------*/
/*
    Purpose : Blocking read from a device

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_read( iod, buf, size, len )

    Input     : x        IWORD      C

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_read(int iod, unsigned char *buf, int size, int *len)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in netlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "netdlib_iod[]"           */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("io_read")               ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device" or "net_open_protocol". We do not allow a call *
 *          to the present procedure with an IOD which has not been          *
 *          allocated using "net_open_device/protocol". We call the          *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one of         */
                    &moidx,&ioidx)   ; /* "net_open_device/protocol"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("io_read")               ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_receive ---------------------------------------------------*/
/*
    Purpose : Gives receive tokens to a network interface driver or a
              protocol.

    Author : JYA                                 Last modification : 11/13/2022

...............................................................................

    Calling example : net_receive( iod, nbtok, size, count )

    Input     : x        IWORD      C

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_receive(int iod, int nbtok, int size, int *count)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in netlib_modd/netlib_iodnb  */
    INT            ioidx             ; /* Index in "netdlib_iod[]"           */
    IWORD          i                 ; /* Loop counter                       */
    UCHAR          *adr              ; /* For "drv_read"                     */
    INT            len, offset, lg   ; /* Returned by "drv_read"             */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_receive")           ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device" or "net_open_protocol". We do not allow a call *
 *          to the present procedure with an IOD which has not been          *
 *          allocated using "net_open_device/protocol". We call the          *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one from       */
                    &moidx,&ioidx)   ; /* "net_open_device/protocol"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


/*****************************************************************************
 * Step 2 : In order to have data flow control, the application have to      *
 * ------   give to the driver receive tokens. One token is one buffer.      *
 *          The applications gives to the driver a number of tokens "nbtok"  *
 *          and the size "size" of the buffers. The application as to send   *
 *          a REQ_READ request to the VMIO automaton. To allocate the "nbtok"*
 *          buffers and send one REQ_READ, we call "drv_read".               *
 *          ---------------------------------------------------------------- *
 *          RESP_READ event will be received after data will be received     *
 *          from the telecomunication line. This may take a long time, or    *
 *          may never occur. So we are not going to unschedule right now     *
 *****************************************************************************/

    adr = HNULL                      ; /* No buffer address, it will be      */
                                       /* allocated by "drv_read"            */
    for (i = 0 ; i LT nbtok ; i++)     /* Loop with the "nbtok" read tokens  */
      BEGIN                            /* to be given to the driver          */
        r = drv_read(iod       ,       /* I/O descriptor for DEV0            */
                     size      ,       /* Maximum number of bytes / buffer   */
                     0         ,       /* Flags                              */
                     &adr      ,       /* Buffer address                     */
                     &len      ,       /* Allocated buffer address           */
                     &offset   ,       /* Offset (unused)                    */
                     &lg       ,       /* Number of bytes read (unused)      */
                     11         )    ; /* Reqid                              */
        if ( r )                       /* If operation failed,               */
          break                      ; /* we stop looping                    */
                                       /*                                    */
        *count += 1                  ; /* Increment the count of tokens      */
      END_FOR

/*****************************************************************************
 * Step 3 : If we have any error, we call the OS halt "vhalt" function       *
 * ------                                                                    *
 *****************************************************************************/

    exit                             : /* Exit label                         */
    if ( r )                           /* If an error has occured            */
      vhalt(0)                       ; /* OS stop if error                   */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_receive")           ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_setval ----------------------------------------------------*/
/*
    Purpose : Set/write a list of tags

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_setval( iod, taglist )

    Input     : x        IWORD      C

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_setval(int iod, const char *taglist)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_setval")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device" or "net_open_protocol". We do not allow a call *
 *          to the present procedure with an IOD which has not been          *
 *          allocated using "net_open_device/protocol". We call the          *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one from       */
                    &moidx,&ioidx)   ; /* "net_open_device/protocol"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_setval")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_getval ----------------------------------------------------*/
/*
    Purpose : Read, get a list of tags

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_getval ( iod, taglist, sz )

    Input     : x        IWORD      C

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_getval(int iod, char *taglist, int sz)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_getval")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device" or "net_open_protocol". We do not allow a call *
 *          to the present procedure with an IOD which has not been          *
 *          allocated using "net_open_device/protocol". We call the          *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one from       */
                    &moidx,&ioidx)   ; /* "net_open_device/protocol"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_getval")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure net_nodeid_req ------------------------------------------------*/
/*
    Purpose : Sends to a master a request to allocate a node identifier

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_nodeid_req( iod, data, len )

    Input     : iod      INT        Network or driver IO identifer
                data     UCHAR      Randomized data used as a key
                len      INT        Data size in bytes

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_nodeid_req(int iod, const unsigned char *data, int len)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_nodeid_req")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device" or "net_open_protocol". We do not allow a call *
 *          to the present procedure with an IOD which has not been          *
 *          allocated using "net_open_device/protocol". We call the          *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one from       */
                    &moidx,&ioidx)   ; /* "net_open_device/protocol"         */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */


    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_nodeid_req")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_nodeid_resp -----------------------------------------------*/
/*
    Purpose : Sends a response holding an allocated node identifier

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_nodeid_resp( iod , data, len, nodeid )

    Input     : iod      INT        Network or driver IO identifer
                data     UCHAR      Randomized data used as a key
                len      INT        Data size in bytes
                nodeid   INT        Allocated node Identifier

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_nodeid_resp(int iod, const unsigned char *data, 
                                 int len, int nodeid                )

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            moidx             ; /* Indew in iodlib_modd / iolib_iodnb */
    INT            ioidx             ; /* Index in "iodlib_iod[]"            */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_nodeid_resp")       ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, previously returned by the  *
 * ------   "net_open_device" or "net_open_protocol". We do not allow a call *
 *          to the present procedure with an IOD which has not been          *
 *          allocated using "net_open_device/protocol". We call the          *
 *          "net_chk_iod" subroutine                                         *
 *****************************************************************************/

    r = net_chk_iod(iod, 2,            /* Check that "iod" is one from       */
                    &moidx,&ioidx)   ; /* net_open_device/protocol"          */
                                       /*                                    */
    if ( r )                           /* Exit if "iod" is incorrect with    */
      goto exit                      ; /* the BAD_IOD error code             */

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_nodeid_resp")       ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure net_nodename_req ----------------------------------------------*/
/*
    Purpose : Sends a request to get a node name / serial number

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_nodename_req ( nodeid , name )

    Input     : nodeid   INT        Target node identifier

    Output    : name     CHAR*      Node name

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_nodename_req(int nodeid, const char *name)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_nodename_req")      ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    r = 0                            ;
    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

/*  exit                             : /  Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_nodename_req")      ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_nodename_resp ---------------------------------------------*/
/*
    Purpose : Sends a response holding a node name / serial number

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_nodename_resp( nodeid, name )

    Input     : nodeid   INT        Target node identifier

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_nodename_resp(int nodeid, const char *name)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_nodename_resp")     ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    r = 0                            ;
    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

/*  exit                             : /  Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_nodename_resp")     ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure net_objectid_req ----------------------------------------------*/
/*
    Purpose : Sends a request to allocate an object identifier

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_objectid_req( nodeid, type, objectid )

    Input     : nodeid   INT        Target node identifier
                type     INT        Type of the object

    Output    : objectid INT        Allocated object identifier

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_objectid_req(int nodeid, int type, int *objectid)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_objectid_req")      ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    r = 0                            ;

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

/*  exit                             : /  Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_objectid_req")      ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure net_objectid_resp ---------------------------------------------*/
/*
    Purpose : Sends a response holding an allocated object identifier

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_objectid_resp( nodeid, objectid )

    Input     : nodeid   INT        Target node identifier
                objectid INT        Allocated object identifier

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_objectid_resp(int nodeid, int objectid)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_objectid_resp")     ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    r = 0                            ;

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

/*  exit                             : /  Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_objectid_resp")     ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_send_cmd --------------------------------------------------*/
/*
    Purpose : Sends a command

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_send_cmd( nodeid, objectid, data, len)

    Input     : nodeid   INT        Target node identifier
                objectid INT        Target object id
                data     UCHAR*     Associated data
                len      INT        Data length size in bytes

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_send_cmd(int nodeid, int objectid, 
                              const unsigned char *data, int len)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_send_cmd")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/
    r = 0                            ;

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

/*  exit                             : /  Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_send_cmd")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_send_val --------------------------------------------------*/
/*
    Purpose : Sends a value

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_send_val(nodeid, objectid, data, len)

    Input     : nodeid   INT        Target node identifier
                objectid INT        Source object identifier
                data     UCHAR*     Associated data
                len      INT        Data length in bytes

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE net_send_val(int nodeid, int objectid, 
                              const unsigned char *data, int len)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_send_val")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/
    r = 0                            ;

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

/*  exit                             : /  Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_send_val")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_get_iodidx ------------------------------------------------*/
/*
    Purpose : Subroutine which returns a free entry index in the "netlib_iod[]"
              table. The function simply scans the table and returns the
              first free entry index.

    Author :                                     Last modification : 11/12/2022

...............................................................................

    Calling example : io_get_iodidx ( idx, netiod )

    Input     : -

    Output    : idx      INT*       Index in the "inetlib_iod[]" table
                netiod   Net_iod**  Address of "netlib_iod[idx]"
                valproc  IWORD      0:Success NOT_ENOUGH_MEM: Table full

    Notes : 

...............................................................................
*/

static IWORD  PROCEDURE net_get_iodidx(INT *idx, Net_iod **netiod)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            ioidx             ; /* Index in the "netlib_iod[]" table  */
    Net_iod        *p                ; /* Pointer to "netlib_iod[ioidx]"     */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_get_iodidx")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Scan the "netlib_iod[]" table, looking for the first free        *
 * -------   entry, with a 0 value. We use "ioidx" as the current index      *
 *           value. Is no entry is found, we exit with NOT_ENOUGH_MEM        *
 *****************************************************************************/

    r = 0                            ; /* Let's assume success               */
                                       /*                                    */
    for (ioidx = 0, p = netlib_iod ;   /* We scan the "netlib_iod[]" array,  */
         ioidx LT netlib_nbiod     ;   /* using "iodidx" as the loop and     */
         ioidx ++, p++              )  /* array index                        */
      BEGIN                            /*                                    */
        if ( p->iod EQ 0 )             /* If entry "iodidx" is free, if the  */
          goto exit                  ; /* value is 0, we continue with next  */
      END_FOR                          /* step 2. Else we exit, returning    */
                                       /* the NOT_ENOUGH_MEM error code      */
    r = NOT_ENOUGH_MEM               ; /* Let's assume we will fail          */


/*****************************************************************************
 * Step  2 : We have to return the "idx" value. If we failed we return -1    *
 * --------  else we return the "ioidx" found index value. We also return    *
 *           "p" as "netiod" if we are successful else we reurn HNULL        *
 *****************************************************************************/

    exit                             : /* Procedure exit label               */
                                       /*                                    */
    if ( r EQ 0 )                      /* If we succeeded, we return         */
      BEGIN                            /*                                    */
        *idx    = ioidx              ; /* "ioidx" as "idx" and               */
        *netiod = p                  ; /* "p" as "netiod"                    */
      END_IF                           /*                                    */
    else                               /*                                    */
      BEGIN                            /* If we failed, we return            */
        *idx    = -1                 ; /* -1 as "idx" and                    */
        *netiod = HNULL              ; /* HNULL as "netiod"                  */
      END_ELSE                         /*                                    */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_get_iodidx")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_chk_driver ------------------------------------------------*/
/*
    Purpose : Check that a driver name is known to the present module. This
              module owns a dedicated table "netlib_moddef" which gives the list
              of driver the present module is allowed to use. This list is
              a subset of the list returned by "drv_list_module".

    Author : JYA                                 Last modification : 10/23/2022

...............................................................................

    Calling example : net_chk_driver ( driver, moidx, maj )

    Input     : driver   CHAR*      Driver name we want to check

    Output    : moidx    INT*       Found index in the "iolib_moddef[]" table
                maj      INT*       Foud index in the "vmos_driver" table
                valproc  IWORD      Error code

    Notes : 

...............................................................................
*/

static IWORD  PROCEDURE net_chk_driver(const CHAR *driver, INT *moidx, INT *maj)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD             r              ; /* Return code of the procedure       */
    IWORD             i              ; /* Loop counter for "iolib_moddef[]"  */
    const Mod_def     *pmd           ; /* Pointer to "iolib_moddef[i]"       */
    IWORD             j              ; /* Loop counter for "vmos_driver[]"   */
    const Sys_driver  *vmd           ; /* Pointer to "vmos_driver[j]"        */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_chk_driver")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Search for the "driver" string in the "netlib_moddef[]" table.  *
 * -------   We loop on the table entries, comparing "driver" with the       *
 *           "modname" fields of the "iolib_moddef" entries. If the "driver" *
 *           string is not found, we exit with BAD_NOMP. User gave us an     *
 *           unknown name.                                                   *
 *****************************************************************************/

    r      = BAD_NOMP                ; /* Let's assume we will fail          */
    *moidx = -1                      ; /* Invalid/empty values for both      */
    *maj   = -1                      ; /* "moidx" and "maj"                  */
                                       /*                                    */
    for(i = 0, pmd = netlib_moddef ;   /* We scan the "netlib_nbmodd" entries*/
        i LT netlib_nbmodd         ;   /* of the "netlib_moddef[]"  table    */
        i++, pmd++                  )  /*                                    */
      BEGIN                            /* If the "modname" string of the     */
        if (Strcmp(driver,             /* current "pmd" entry is equal to    */
                   pmd->modname) EQ 0) /* the "driver" string we are looking */
          goto step2                 ; /* for, we stop looping and continue  */
      END_FOR                          /* with next step 2                   */
                                       /*                                    */
    goto exit                        ; /* We failed. We exit with BAD_NOMP   */


/*****************************************************************************
 * Step  2 : Search for the "driver" string in the "vmos_driver[]" table. We *
 * -------   loop on the table entries, we compare "driver" with the "name"  *
 *           field of the "Sys_driver" structure. If the search fails, we    *
 *           have a major issue: The name is in the "iolib_moddef[]" table   *
 *           but not into "vmos_driver", implying the two tables are not     *
 *           coherent, "space_appcont_xyz.c" must be corrected. Indeed, we   *
 *           only take in account "vmos_driver[]" entries corresponding to   *
 *           character drivers, with "typdrv" EQ to DRV_CAR.                 *
 *****************************************************************************/

    step2                            : /* Search "driver" in "vmod_driver[]" */
                                       /*                                    */
    r = -1                           ; /* Let's assume we will fail          */
                                       /*                                    */
    for(j=0; j LT vmos_nbdriver; j++)  /* Scan the "vmos_driver[]", which    */
      BEGIN                            /* is a list of structure addresses   */
        vmd = vmos_driver[j]         ; /* Get the structure address # i      */
                                       /*                                    */
        if (vmd->typdrv NE DRV_CAR)    /* If the structure is not the one    */
          continue                   ; /* of a character driver, skip it     */
                                       /*                                    */
        if(Strcmp(driver,              /* If string "vmd->name" is the one   */
                  vmd->name) EQ 0)     /* we are looking for, we are done,   */
          goto step3                 ; /* we exit for the loop and continue  */
      END_FOR                          /* with next step 3                   */
                                       /*                                    */
    vhalt(0)                         ; /* We failed because of incorrect     */
    goto exit                        ; /* data in "space_appcont_xyz.c"      */


/*****************************************************************************
 * Step  3 : We are OK, the "driver" string has been found in both the       *
 * -------   "iolib_moddef[]" and the "vmos_driver[]" tables:                *
 *           - We clear the "r" function return code - No error              *
 *           - We set the "moidx" index in "iolib_moddef[]" to "i"           *
 *           - We set the "maj" index in "vmos_driver[]" to "j"              * 
 *****************************************************************************/

    step3                            : /* We are OK                          */
                                       /*                                    */
    r      = 0                       ; /* We clear the error code            */
    *moidx = i                       ; /* Index in the "iolib_moddef[]" table*/
    *maj   = j                       ; /* Index in the "vmos_driver[]" table */

    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_chk_driver")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure net_chk_iod ---------------------------------------------------*/
/*
    Purpose : Checks the an IOD is one allocated by the present module. We
              do that because user may allocate other IOD's, by-passing the
              present module. So, here we check that an IOD has been allocated
              by the present module.

    Author : JYA                                 Last modification : 10/22/2022

...............................................................................

    Calling example : io_chk_iod(iod, typ, moidx, ioidx )

    Input     : iod      INT        IOD to be checked
                typ      INT        IOD type 0:device 1:subchan 2:any

    Output    : moidx    INT        Index in the "iolib_modd[]" table
                ioidx    INT        Index in the "iolib_iod[]" table
                valproc  IWORD      0:Sucess BAD_IOD:Not a "vmk_iolib" IOD

    Notes : - "typ" may be 0:device IOD only 1:subchannel IOD only 2:Any

...............................................................................
*/

IWORD  PROCEDURE net_chk_iod(INT iod, INT typ, INT *moidx, INT *ioidx)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    INT            maj               ; /* Major number << 16                 */
    IWORD          i                 ; /* Loop counter with MODD's           */
    IWORD          j                 ; /* Loop counter with IOD's            */
    Net_iod        *p                ; /* Address of "netlib_iod[j]          */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_chk_iod")           ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Initialization of local variables:                              *
 * -------   - We set the "r" local variable, indicating that "iod" is none  *
 *             of ours.                                                      *
 *           - We set the the -1 empty value both "moidx" and "ioidx"        *
 *****************************************************************************/

    r      = BAD_IOD                 ; /* Let'a assume we will fail          */
    *moidx =                           /* In case of error, we return those  */
    *ioidx = -1                      ; /* two -1 empty values                */

/*****************************************************************************
 * Step  2 : The present function returns the "moidx" index. In order to     *
 * -------   retreive it, we search for the major number :                   *
 *           - We extract bits b16..23 of "iod", which are the "maj" major   *
 *             number value                                                  *
 *           - Then we loop with the entries of the "netlib_modd[]" table,   *
 *             which holds the MODD's that are known to the present module,  *
 *             we are happy if we found one MODD that holds in bits b16..23  *
 *             the "maj" major number we are looking for, we stop looping    *
 *****************************************************************************/

    maj = iod AND 0xFF0000           ; /* Extract the major number field     */
                                       /* value, bits b16..23                */
    for (i=0; i LT netlib_nbmodd; i++) /* We loop on the MODD's of the       */
      if((netlib_modd[i] AND 0xFF000)  /* "netlib_modd[]" table, looking for */
          EQ maj                    )  /* the "maj" major number in bits     */
        goto step3                   ; /* b16..23 of the MODD                */
                                       /*                                    */
    goto exit                        ; /* We failed, exit with BAD_IOD       */


/*****************************************************************************
 * Step  3 : Retreive the "ioidx" index value, which is the index in the     *
 * -------   "netlib_iod[]" table of IOD's. We simply loop on the entries of *
 *           this table of INT's, searching for "iod", we stop looping when  *
 *           we have found it.                                               *
 *****************************************************************************/

    step3                            : /* Search for the "iod" value         */
                                       /*                                    */
    for (j = 0, p = netlib_iod ;       /* We loop on the entries of the      */
         j LT netlib_nbiod     ;       /* "netlib_iod[]" table,              */
         j++, p++                )     /*                                    */
      if (p->iod EQ iod)               /* until we find the "iod" value we   */
        goto step4                  ;  /* are looking for                    */
                                       /*                                    */
    goto exit                        ; /* We failed, exit with BAD_IOD       */

/*****************************************************************************
 * Step  4 : Compare the IOD type against the "typ" awaited type:            *
 * --------  - if "typ" is 0, we are waiting for a device IOD with a 0xFF    *
 *             low-order byte, if this is not 0xFF we have an error          *
 *           - if "typ" is 1, we are waiting for a sub-channel IOD, we exit  *
 *             with BAD_IOD if the lower byte is 0xFF (device IOD)           *
 *           - if "typ" is 2 (oar any other value), we are OK and contine    *
 *****************************************************************************/

    step4                            : /* Check channel or sub-channel IOD   */
    iod &= 0xFF                      ; /* Keep the low-order byte, which is  */
                                       /* the sub-channel number of 0xFF     */
    if (typ EQ 0)                      /* If we are waiting for a device IOD */
      BEGIN                            /* there is no channel number but 0xFF*/
        if (iod NE 0xFF) goto exit   ; /* If channel number is not the 0xFF  */
      END_IF                           /* empty value, exit with BAD_IOD     */
                                       /*                                    */
    else if (typ EQ 1)                 /* If we are waiting for a sub-channel*/
      BEGIN                            /* IOD (sub-channel number not 0xFF)  */
        if (iod EQ 0xFF) goto exit   ; /* We exit with BAD_IOD if channel #  */
      END_ELSE                         /* is the 0xFF empty value/number     */


/*****************************************************************************
 * Step  5 : We are successful, "i" is the index in "netlib_modd[] and "j"   *
 * -------   the index in "netlib_iod[]". We return "i" as "moidx" and "j"   *
 *           as "ioidx".                                                     *
 *****************************************************************************/

    *moidx = i                       ; /* Return the "iolib_moidx" index     */
    *ioidx = j                       ; /* Return the "iolib_iod" index       */
    r      = 0                       ; /* No more error                      */

    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_chk_iod")           ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure net_chk_nodeid ------------------------------------------------*/
/*
    Purpose : Checks that a node identifier is correct.

    Author : JYA                                 Last modification : 11/12/2022

...............................................................................

    Calling example : net_chk_nodeid( nodeid, moidx, ioidx)

    Input     : nodeid   INT        Node ID to be checked

    Output    : moidx    INT*       Index in the "netlib_modd[]" table
                ioidx    INT*       Index in the "netlib_iod[]" table
                valproc  IWORD      0:Sucess BAD_IOD:Not a "tsk_iolib" IOD


    Notes : 

...............................................................................
*/

static IWORD  PROCEDURE net_chk_nodeid(INT nodeid,INT moidx,INT ioidx,INT noidx)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("net_chk_nodeid")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    r = 0                            ;

    ;                                  /*                                    */
    ;                                  /*                                    */
    ;                                  /*                                    */

/*  exit                             : /  Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("net_chk_nodeid")        ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */
