#!  /bin/sh
#   HyperPanel OS environment           File system "shells/hyperpanel.sh"
#   Author : FA                         Last modification : 09/06/2023
#
#   Standard environment - VMIO/VMIO - Linux.
#   Syntax : Called by environment files.
#

# HyperPanel local environment ------------------------------------------------

  export hobj=$hlocal/$hboard/obj                        # Objects location
  export hexe=$hlocal/$hboard/exe                        # Executables location
  export hutest=$hlocal/$hboard/utest                    # Tests exe location


# HyperPanel naviguation alias ------------------------------------------------

  alias  obj='cd $hobj'                                  # Object files
  alias  exe='cd $hexe'                                  # Executables files
  alias  utest='cd $hutest'                              # Unitary tests
  alias  hhome='cd $hhome'                               # Home directory
  alias  hlocal='cd $hlocal'                             # Local directory
  alias  include='cd $hhome/os/include'                  # Includes files
  alias  shells='cd $hhome/shells'                       # /bin/sh Shell scripts
  alias  pkg='cd $hhome/os/package'                      # Packages definition
                                                         #
  function boards()   { cd $hhome/boards/$1       ; }    # Boards files
  function hcomp()    { cd $hhome/hcomp/$1        ; }    # Compiler files
  function boards()   { cd $hhome/boards/$1       ; }    # Boards files
  function hlib()     { cd $hhome/lib/$1          ; }    # Libraries
  function os()       { cd $hhome/os/$1           ; }    # OS source code
  function com()      { cd $hhome/os/com/$1       ; }    # Common source files
  function hal()      { cd $hhome/os/hal/$1       ; }    # Hardware Abstraction
  function vmio()     { cd $hhome/os/vmio/$1      ; }    # VMIO source files
  function vmk()      { cd $hhome/os/vmk/$1       ; }    # VMK  source files
  function vmos()     { cd $hhome/os/vmos/$1      ; }    # VMOS source files
  function tools()    { cd $hhome/tools/$1        ; }    # Tools modules
  function user()     { cd $hhome/user/$1         ; }    # User modules
                                                         #
  function ios()      { cd /mnt/hyperpanel/ios/$1 ; }    # Transfer directory
                                                         #
  alias hcm4='clear; cd $hexe ;                     \
          python3 $hhome/user/hsimu/hcm4/hcm4.py'        # HCM4 simulator

  alias pytest='clear; cd $hexe ;                   \
          python3 $hhome/user/hsimu/pytest/pytest.py'    # Python tests

# Alias -----------------------------------------------------------------------

  alias  hgdb='$hgdb'                                    # Debugger
  alias  dir='ls -alG'                                   # "dir" command


# Shell configuration ---------------------------------------------------------

  umask 0033                                             # Default file att
  set    filec                                           # Filename completion
                                                         #
  export LESS="-FRnX"                                    # "less" default opt
  export PAGER="less"                                    # Default pager
  export EDITOR="vim"                                    # Default editor
  export HISTSIZE=1000                                   # Historical size
                                                         #
  if [ $UID = 0 ]; then                                  # Root case:
    export PS1="$hvm \w #> "                             # Set prompt
  else                                                   # Other cases:
    export PS1="$hvm \w >> "                             # Set prompt
  fi                                                     #

# Trace command ---------------------------------------------------------------

tip() {
  brate=115200
  if [ $# -lt 1 ]; then echo "Please provide TTY interface."; return -1; fi
  case "$1" in
    -*) brate=${1:1}; shift ;;
  esac
  if [ $# -ne 1 ]; then return -2; fi
  socat - "$1",raw,b${brate}
}


# Default directory -----------------------------------------------------------

  cd     $hhome                                          # Default directory.
